@@ -2,7 +2,9 @@ import os
 import sys
 import threading
 from .call import Call
+from .credentials import Credentials
 import ruamel.yaml
+from osc_sdk_python import __version__
 
 type_mapping = {'boolean': 'bool',
                 'string': 'str',
@@ -71,7 +73,8 @@ class OutscaleGateway:
         self.call = Call(logger=self.log, **kwargs)
 
     def update_credentials(self, region=None, profile=None, access_key=None,
-                           secret_key=None, email=None, password=None):
+                           secret_key=None, email=None, password=None,
+                           x509_client_cert=None, proxy=None):
         """
         destroy and create a new credential map use for each call.
         so you can change your ak/sk, region without having to recreate the whole Gateway
@@ -79,7 +82,23 @@ class OutscaleGateway:
         example: just updating the password, without renter the login will fail
         """
         self.call.update_credentials(region=region, profile=profile, access_key=access_key,
-                                     secret_key=secret_key, email=email, password=password)
+                                     secret_key=secret_key, email=email, password=password,
+                                     x509_client_cert=x509_client_cert, proxy=x509_client_cert)
+
+    def access_key(self):
+        return Credentials(**self.call.credentials).access_key
+
+    def secret_key(self):
+        return Credentials(**self.call.credentials).secret_key
+
+    def region(self):
+        return Credentials(**self.call.credentials).region
+
+    def email(self):
+        return Credentials(**self.call.credentials).email
+
+    def password(self):
+        return Credentials(**self.call.credentials).password
 
     def _convert(self, input_file):
         structure = {}
@@ -140,12 +159,14 @@ class OutscaleGateway:
         input_parameters = set(params)
         different_parameters = list(input_parameters.difference(set(structure_parameters)))
         if different_parameters:
-            raise ParameterNotValid("""{}. Available parameters are: {}.""".format(', '.join(different_parameters),
-                                                                                   ', '.join(structure_parameters)))
+            raise ParameterNotValid("""{}. Available parameters on sdk: {} api: {} are: {}.""".format(
+                ', '.join(different_parameters),
+                __version__, self.api_version,
+                ', '.join(structure_parameters)))
 
     def _check(self, action_name, **params):
         if action_name not in self.gateway_structure:
-            raise ActionNotExists('Action {} does not exists'.format(action_name))
+            raise ActionNotExists('Action {} does not exists for python sdk: {} with api: {}'.format(action_name, __version__, self.api_version))
         self._check_parameters_valid(action_name, params)
         self._check_parameters_required(self.gateway_structure[action_name], params)
         self._check_parameters_type(self.gateway_structure[action_name], params)
@@ -158,16 +179,16 @@ class OutscaleGateway:
         """
         return {key: value for key, value in params.items() if value is not None}
 
-    def _action(self, **kwargs):
-        kwargs = self._remove_none_parameters(**kwargs)
-        self._check(self.action_name, **kwargs)
-        result = self.call.api(self.action_name,**kwargs)
-        self.action_name = None
-        return result
+    def _get_action(self, action_name):
+        def action(**kwargs):
+            kwargs = self._remove_none_parameters(**kwargs)
+            self._check(action_name, **kwargs)
+            result = self.call.api(action_name,**kwargs)
+            return result
+        return action
 
     def __getattr__(self, attr):
-        self.action_name = attr
-        return self._action
+        return self._get_action(attr)
 
     def raw(self, action_name, **kwargs):
         return self.call.api(action_name, **kwargs)
